@@ -6,8 +6,10 @@ from .brain_data import Brain_Data, Groupby
 from .adjacency import Adjacency
 from .design_matrix import Design_Matrix, Design_Matrix_Series
 
-__all__ = ['Brain_Data',
-           'Adjacency',
-           'Groupby',
-           'Design_Matrix',
-           'Design_Matrix_Series']
+__all__ = [
+    "Brain_Data",
+    "Adjacency",
+    "Groupby",
+    "Design_Matrix",
+    "Design_Matrix_Series",
+]
