@@ -37,7 +37,9 @@ import os
 
 # Optional dependencies
 ipywidgets = attempt_to_import(
-    "ipywidgets", name="ipywidgets", fromlist=["interact", "fixed", "widgets", "BoundedFloatText", "BoundedIntText"]
+    "ipywidgets",
+    name="ipywidgets",
+    fromlist=["interact", "fixed", "widgets", "BoundedFloatText", "BoundedIntText"],
 )
 
 
@@ -47,7 +49,7 @@ def plot_interactive_brain(
     surface=False,
     percentile_threshold=False,
     anatomical=None,
-    **kwargs
+    **kwargs,
 ):
     """
     This function leverages nilearn's new javascript based brain viewer functions to create interactive plotting functionality.
@@ -72,7 +74,9 @@ def plot_interactive_brain(
         if threshold[-1] != "%":
             raise ValueError("Starting threshold provided as string must end in '%'")
         percentile_threshold = True
-        warnings.warn("Percentile thresholding ignores brain mask. Results are likely more liberal than you expect (e.g. with non-interactive plotting)!")
+        warnings.warn(
+            "Percentile thresholding ignores brain mask. Results are likely more liberal than you expect (e.g. with non-interactive plotting)!"
+        )
         threshold = int(threshold[:-1])
 
     if len(brain.shape()) == 2:
@@ -108,7 +112,7 @@ def plot_interactive_brain(
         percentile_threshold=percentile_threshold,
         surface=surface,
         anatomical=ipywidgets.fixed(anatomical),
-        **kwargs
+        **kwargs,
     )
 
 
@@ -208,7 +212,7 @@ def plot_t_brain(
             colorbar=True,
             cmap=cmap,
             plot_abs=False,
-            **kwargs
+            **kwargs,
         )
         for v, c in zip(views, cut_coords):
             plot_stat_map(
@@ -217,7 +221,7 @@ def plot_t_brain(
                 display_mode=v,
                 cmap=cmap,
                 bg_img=resolve_mni_path(MNI_Template)["brain"],
-                **kwargs
+                **kwargs,
             )
     elif how == "glass":
         plot_glass_brain(
@@ -226,7 +230,7 @@ def plot_t_brain(
             colorbar=True,
             cmap=cmap,
             plot_abs=False,
-            **kwargs
+            **kwargs,
         )
     elif how == "mni":
         for v, c in zip(views, cut_coords):
@@ -236,7 +240,7 @@ def plot_t_brain(
                 display_mode=v,
                 cmap=cmap,
                 bg_img=resolve_mni_path(MNI_Template)["brain"],
-                **kwargs
+                **kwargs,
             )
     del obj
     del out
@@ -299,7 +303,7 @@ def plot_brain(objIn, how="full", thr_upper=None, thr_lower=None, save=False, **
             colorbar=True,
             cmap=cmap,
             plot_abs=False,
-            **kwargs
+            **kwargs,
         )
         if save:
             plt.savefig(glass_save, bbox_inches="tight")
@@ -310,7 +314,7 @@ def plot_brain(objIn, how="full", thr_upper=None, thr_lower=None, save=False, **
                 display_mode=v,
                 cmap=cmap,
                 bg_img=resolve_mni_path(MNI_Template)["brain"],
-                **kwargs
+                **kwargs,
             )
             if save:
                 plt.savefig(savefile, bbox_inches="tight")
@@ -321,7 +325,7 @@ def plot_brain(objIn, how="full", thr_upper=None, thr_lower=None, save=False, **
             colorbar=True,
             cmap=cmap,
             plot_abs=False,
-            **kwargs
+            **kwargs,
         )
         if save:
             plt.savefig(glass_save, bbox_inches="tight")
@@ -333,7 +337,7 @@ def plot_brain(objIn, how="full", thr_upper=None, thr_lower=None, save=False, **
                 display_mode=v,
                 cmap=cmap,
                 bg_img=resolve_mni_path(MNI_Template)["brain"],
-                **kwargs
+                **kwargs,
             )
             if save:
                 plt.savefig(savefile, bbox_inches="tight")
@@ -342,7 +346,7 @@ def plot_brain(objIn, how="full", thr_upper=None, thr_lower=None, save=False, **
 
 
 def dist_from_hyperplane_plot(stats_output):
-    """ Plot SVM Classification Distance from Hyperplane
+    """Plot SVM Classification Distance from Hyperplane
 
     Args:
         stats_output: a pandas file with prediction output
@@ -371,11 +375,11 @@ def dist_from_hyperplane_plot(stats_output):
     plt.xlabel("Subject", fontsize=16)
     plt.ylabel("Distance from Hyperplane", fontsize=16)
     plt.title("Classification", fontsize=18)
-    return 
+    return
 
 
 def scatterplot(stats_output):
-    """ Plot Prediction Scatterplot
+    """Plot Prediction Scatterplot
 
     Args:
         stats_output: a pandas file with prediction output
@@ -392,11 +396,11 @@ def scatterplot(stats_output):
     plt.xlabel("Y", fontsize=16)
     plt.ylabel("Predicted Value", fontsize=16)
     plt.title("Prediction", fontsize=18)
-    return 
+    return
 
 
 def probability_plot(stats_output):
-    """ Plot Classification Probability
+    """Plot Classification Probability
 
     Args:
         stats_output: a pandas file with prediction output
@@ -412,7 +416,7 @@ def probability_plot(stats_output):
     plt.xlabel("Y", fontsize=16)
     plt.ylabel("Predicted Probability", fontsize=16)
     plt.title("Prediction", fontsize=18)
-    return 
+    return
 
     # # and plot the result
     # plt.figure(1, figsize=(4, 3))
@@ -427,7 +431,7 @@ def probability_plot(stats_output):
 
 
 def roc_plot(fpr, tpr):
-    """ Plot 1-Specificity by Sensitivity
+    """Plot 1-Specificity by Sensitivity
 
     Args:
         fpr: false positive rate from Roc.calculate
@@ -448,7 +452,7 @@ def roc_plot(fpr, tpr):
 
 
 def plot_stacked_adjacency(adjacency1, adjacency2, normalize=True, **kwargs):
-    """ Create stacked adjacency to illustrate similarity.
+    """Create stacked adjacency to illustrate similarity.
 
     Args:
         matrix1:  Adjacency instance 1
@@ -483,9 +487,9 @@ def plot_mean_label_distance(
     permutation_test=False,
     n_permute=5000,
     fontsize=18,
-    **kwargs
+    **kwargs,
 ):
-    """ Create a violin plot indicating within and between label distance.
+    """Create a violin plot indicating within and between label distance.
 
     Args:
         distance:  pandas dataframe of distance
@@ -531,7 +535,7 @@ def plot_mean_label_distance(
         inner="quartile",
         palette={"Within": "lightskyblue", "Between": "red"},
         ax=ax,
-        **kwargs
+        **kwargs,
     )
     f.set_ylabel("Average Distance", fontsize=fontsize)
     f.set_title("Average Group Distance", fontsize=fontsize)
@@ -554,24 +558,24 @@ def plot_between_label_distance(
     permutation_test=True,
     n_permute=5000,
     fontsize=18,
-    **kwargs
+    **kwargs,
 ):
-    """ Create a heatmap indicating average between label distance
+    """Create a heatmap indicating average between label distance
 
 
-        Args:
-            distance: (pandas dataframe) brain_distance matrix
-            labels: (pandas dataframe) group labels
-            ax: axis to plot (default=None)
-            permutation_test: (boolean)
-            n_permute: (int) number of samples for permuation test
-            fontsize: (int) size of font for plot
-        Returns:
-            f: heatmap
-            out: pandas dataframe of pairwise distance between conditions
-            within_dist_out: average pairwise distance matrix
-            mn_dist_out: (optional if permutation_test=True) average difference in distance between conditions
-            p_dist_out: (optional if permutation_test=True) p-value for difference in distance between conditions
+    Args:
+        distance: (pandas dataframe) brain_distance matrix
+        labels: (pandas dataframe) group labels
+        ax: axis to plot (default=None)
+        permutation_test: (boolean)
+        n_permute: (int) number of samples for permuation test
+        fontsize: (int) size of font for plot
+    Returns:
+        f: heatmap
+        out: pandas dataframe of pairwise distance between conditions
+        within_dist_out: average pairwise distance matrix
+        mn_dist_out: (optional if permutation_test=True) average difference in distance between conditions
+        p_dist_out: (optional if permutation_test=True) p-value for difference in distance between conditions
     """
 
     labels = np.unique(np.array(labels))
@@ -652,23 +656,23 @@ def plot_between_label_distance(
 def plot_silhouette(
     distance, labels, ax=None, permutation_test=True, n_permute=5000, **kwargs
 ):
-    """ Create a silhouette plot indicating between relative to within label distance
+    """Create a silhouette plot indicating between relative to within label distance
 
-        Args:
-            distance: (pandas dataframe) brain_distance matrix
-            labels: (pandas dataframe) group labels
-            ax: axis to plot (default=None)
-            permutation_test: (boolean)
-            n_permute: (int) number of samples for permuation test
-        Optional keyword args:
-            figsize: (list) dimensions of silhouette plot
-            colors: (list) color triplets for silhouettes. Length must equal number of unique labels
-        Returns:
-            # f: heatmap
-            # out: pandas dataframe of pairwise distance between conditions
-            # within_dist_out: average pairwise distance matrix
-            # mn_dist_out: (optional if permutation_test=True) average difference in distance between conditions
-            # p_dist_out: (optional if permutation_test=True) p-value for difference in distance between conditions
+    Args:
+        distance: (pandas dataframe) brain_distance matrix
+        labels: (pandas dataframe) group labels
+        ax: axis to plot (default=None)
+        permutation_test: (boolean)
+        n_permute: (int) number of samples for permuation test
+    Optional keyword args:
+        figsize: (list) dimensions of silhouette plot
+        colors: (list) color triplets for silhouettes. Length must equal number of unique labels
+    Returns:
+        # f: heatmap
+        # out: pandas dataframe of pairwise distance between conditions
+        # within_dist_out: average pairwise distance matrix
+        # mn_dist_out: (optional if permutation_test=True) average difference in distance between conditions
+        # p_dist_out: (optional if permutation_test=True) p-value for difference in distance between conditions
     """
 
     # Define label set
@@ -756,13 +760,14 @@ def plot_silhouette(
     else:
         return
 
+
 def component_viewer(output, tr=2.0):
-    ''' This a function to interactively view the results of a decomposition analysis
+    """This a function to interactively view the results of a decomposition analysis
 
     Args:
         output: (dict) output dictionary from running Brain_data.decompose()
         tr: (float) repetition time of data
-    '''
+    """
 
     if ipywidgets is None:
         raise ImportError(
@@ -770,43 +775,65 @@ def component_viewer(output, tr=2.0):
         )
 
     def component_inspector(component, threshold):
-        '''This a function to be used with ipywidgets to interactively view a decomposition analysis
+        """This a function to be used with ipywidgets to interactively view a decomposition analysis
 
-            Make sure you have tr and output assigned to variables.
+        Make sure you have tr and output assigned to variables.
 
-            Example:
+        Example:
 
-                from ipywidgets import BoundedFloatText, BoundedIntText
-                from ipywidgets import interact
+            from ipywidgets import BoundedFloatText, BoundedIntText
+            from ipywidgets import interact
 
-                tr = 2.4
-                output = data_filtered_smoothed.decompose(algorithm='ica', n_components=30, axis='images', whiten=True)
+            tr = 2.4
+            output = data_filtered_smoothed.decompose(algorithm='ica', n_components=30, axis='images', whiten=True)
 
-                interact(component_inspector, component=BoundedIntText(description='Component', value=0, min=0, max=len(output['components'])-1),
-                      threshold=BoundedFloatText(description='Threshold', value=2.0, min=0, max=4, step=.1))
+            interact(component_inspector, component=BoundedIntText(description='Component', value=0, min=0, max=len(output['components'])-1),
+                  threshold=BoundedFloatText(description='Threshold', value=2.0, min=0, max=4, step=.1))
 
-        '''
-        _, ax = plt.subplots(nrows=3, figsize=(12,8))
-        thresholded = (output['components'][component] - output['components'][component].mean())*(1/output['components'][component].std())
+        """
+        _, ax = plt.subplots(nrows=3, figsize=(12, 8))
+        thresholded = (
+            output["components"][component] - output["components"][component].mean()
+        ) * (1 / output["components"][component].std())
         thresholded.data[np.abs(thresholded.data) <= threshold] = 0
-        plot_stat_map(thresholded.to_nifti(), cut_coords=range(-40, 70, 10),
-                      display_mode='z', black_bg=True, colorbar=True, annotate=False,
-                      draw_cross=False, axes=ax[0])
-        if isinstance(output['decomposition_object'], (sklearn.decomposition.PCA)):
-            var_exp = output['decomposition_object'].explained_variance_ratio_[component]
-            ax[0].set_title(f"Component: {component}/{len(output['components'])}, Variance Explained: {var_exp:2.2}", fontsize=18)
+        plot_stat_map(
+            thresholded.to_nifti(),
+            cut_coords=range(-40, 70, 10),
+            display_mode="z",
+            black_bg=True,
+            colorbar=True,
+            annotate=False,
+            draw_cross=False,
+            axes=ax[0],
+        )
+        if isinstance(output["decomposition_object"], (sklearn.decomposition.PCA)):
+            var_exp = output["decomposition_object"].explained_variance_ratio_[
+                component
+            ]
+            ax[0].set_title(
+                f"Component: {component}/{len(output['components'])}, Variance Explained: {var_exp:2.2}",
+                fontsize=18,
+            )
         else:
-            ax[0].set_title(f"Component: {component}/{len(output['components'])}", fontsize=18)
+            ax[0].set_title(
+                f"Component: {component}/{len(output['components'])}", fontsize=18
+            )
 
-        ax[1].plot(output['weights'][:, component], linewidth=2, color='red')
-        ax[1].set_ylabel('Intensity (AU)', fontsize=18)
-        ax[1].set_title(f'Timecourse (TR={tr})', fontsize=16)
-        y = fft(output['weights'][:, component])
+        ax[1].plot(output["weights"][:, component], linewidth=2, color="red")
+        ax[1].set_ylabel("Intensity (AU)", fontsize=18)
+        ax[1].set_title(f"Timecourse (TR={tr})", fontsize=16)
+        y = fft(output["weights"][:, component])
         f = fftfreq(len(y), d=tr)
-        ax[2].plot(f[f > 0], np.abs(y)[f > 0]**2)
-        ax[2].set_ylabel('Power', fontsize=18)
-        ax[2].set_xlabel('Frequency (Hz)', fontsize=16)
-
-    ipywidgets.interact(component_inspector, component=ipywidgets.BoundedIntText(description='Component', value=0, min=0, max=len(output['components'])-1),
-              threshold=ipywidgets.BoundedFloatText(description='Threshold', value=2.0, min=0, max=4, step=.1))
+        ax[2].plot(f[f > 0], np.abs(y)[f > 0] ** 2)
+        ax[2].set_ylabel("Power", fontsize=18)
+        ax[2].set_xlabel("Frequency (Hz)", fontsize=16)
 
+    ipywidgets.interact(
+        component_inspector,
+        component=ipywidgets.BoundedIntText(
+            description="Component", value=0, min=0, max=len(output["components"]) - 1
+        ),
+        threshold=ipywidgets.BoundedFloatText(
+            description="Threshold", value=2.0, min=0, max=4, step=0.1
+        ),
+    )
